// This algorithm is from the book "GPU GEM2"

uniform sampler2DShadow texture0;	// shadowMap
uniform samplerCube texture1;		// lightMap
uniform sampler3D texture2;		// jitterMap

varying vec4 ambient;

void main (void)
{		
	int totalSamples = 64;
	
	vec4 shadowMapCoord = gl_TexCoord[0];
	float filterSize = .48;

	vec2 scaleJitterxy = vec2(.5, .5);	// Scales the jitter offset, too smaller number creates visual sample squares
	vec3 jitterCoord = vec3(gl_FragCoord.xy * scaleJitterxy, 0.0);
	
	float shadow = 0.0;
	
	for (int i = 0; i < totalSamples/2; i++)
	{
		vec4 offset = (2.0 * texture3D(texture2, jitterCoord)) - 1.0;
		jitterCoord.z += 1.0/32.0;	// Steps down thru 3D texture column
	
		shadowMapCoord.xy = offset.xy * filterSize + gl_TexCoord[0].xy;
		shadow += shadow2DProj(texture0, shadowMapCoord);
		shadowMapCoord.xy = offset.zw * filterSize + gl_TexCoord[0].xy;
		shadow += shadow2DProj(texture0, shadowMapCoord);	
	}
	shadow /= totalSamples;	// Percentage in shadow

	float diffuseShadow = clamp(shadow, 0.2, 1.0);
	vec4 specular = textureCube(texture1, gl_TexCoord[1].xyz);
	
	gl_FragColor = ambient + 
		       gl_Color * diffuseShadow +
		       specular;			
}


